@@ -1,22 +1,28 @@
-"""This module contains the main logic for tootbot."""
+"""Main logic for tootbot."""
 import argparse
 import asyncio
 import logging
 import math
+import sys
 import time
 
 import aiohttp
 import arrow
+import asyncprawcore
+from imgurpython.helpers.error import ImgurClientError
+from minimal_activitypub.client_2_server import ActivityPubError
+from minimal_activitypub.client_2_server import ApiError
 from outdated import check_outdated
 from tqdm import tqdm
 
+from . import FATAL_TOOTBOT_ERROR
+from . import PROGRESS_BAR_FORMAT
 from . import __display_name__
 from . import __package_name__
 from . import __version__
-from . import PROGRESS_BAR_FORMAT
-from .collect import get_secrets
 from .collect import LinkedMediaHelper
 from .collect import RedditHelper
+from .collect import get_secrets
 from .control import Configuration
 from .control import PostRecorder
 from .monitoring import HealthChecks
@@ -26,8 +32,8 @@ logger = logging.getLogger(__display_name__)
 logger.setLevel(logging.DEBUG)
 
 
-async def main() -> None:
-    """Main / Overall Logic of tootbot.
+async def main() -> None:  # noqa: C901, PLR0915
+    """Start tootbot.
 
     :param: None
     :return: None
@@ -61,14 +67,45 @@ async def main() -> None:
     print(f"Welcome to {__display_name__} ({__version__})")
     check_updates()
 
-    secrets = await get_secrets(config_dir=config_dir)
-    secrets["mastodon"] = await MastodonPublisher.get_secrets(
-        mastodon_domain=config.mastodon_config.domain,
-        config_dir=config_dir,
-    )
+    try:
+        secrets = await get_secrets(config_dir=config_dir)
+    except ImgurClientError as imgur_error:
+        logger.error("Error on creating ImgurClient: %s", imgur_error)
+        logger.error(FATAL_TOOTBOT_ERROR)
+        await config.bot.post_recorder.close_db()
+        sys.exit(1)
+    except asyncprawcore.AsyncPrawcoreException as reddit_exception:
+        logger.error("Error while logging into Reddit: %s", reddit_exception)
+        logger.error(FATAL_TOOTBOT_ERROR)
+        await config.bot.post_recorder.close_db()
+        sys.exit(1)
+
+    try:
+        secrets["mastodon"] = await MastodonPublisher.get_secrets(
+            mastodon_domain=config.mastodon_config.domain,
+            config_dir=config_dir,
+        )
+    except (ActivityPubError, ApiError) as error:
+        logger.error("Error while logging into Mastodon: %s", error)
+        logger.error("Tootbot cannot continue, now shutting down")
+        await config.bot.post_recorder.close_db()
+        sys.exit(1)
 
     session = aiohttp.ClientSession()
 
+    try:
+        mastodon_publisher = await MastodonPublisher.initialise(
+            config=config,
+            session=session,
+            secrets=secrets["mastodon"],
+        )
+    except (ActivityPubError, ApiError) as error:
+        logger.error("Error while logging into Mastodon: %s", error)
+        logger.error("Tootbot cannot continue, now shutting down")
+        await session.close()
+        await config.bot.post_recorder.close_db()
+        sys.exit(1)
+
     title = "Setting up shop "
     with tqdm(
         desc=f"{title:.<60}",
@@ -77,24 +114,23 @@ async def main() -> None:
         ncols=120,
         bar_format=PROGRESS_BAR_FORMAT,
     ) as progress_bar:
-        mastodon_publisher = await MastodonPublisher.initialise(
-            config=config,
-            session=session,
-            secrets=secrets["mastodon"],
-        )
-        progress_bar.update(0.4120)
-
         healthcheck = HealthChecks(config=config)
         progress_bar.update(0.0002)
 
         reddit = RedditHelper(config=config, api_secret=secrets["reddit"])
         progress_bar.update(0.0008)
 
-        media_helper = LinkedMediaHelper(
-            config=config,
-            imgur_secrets=secrets["imgur"],
-        )
-        progress_bar.update(1 - 0.4130)
+        try:
+            media_helper = LinkedMediaHelper(
+                config=config,
+                imgur_secrets=secrets["imgur"],
+            )
+            progress_bar.update(0.999)
+        except ImgurClientError as imgur_error:
+            logger.error("Error on creating ImgurClient: %s", imgur_error)
+            logger.error(FATAL_TOOTBOT_ERROR)
+            await config.bot.post_recorder.close_db()
+            sys.exit(1)
 
     now_timestamp = arrow.now().timestamp()
     last_post_ts = await config.bot.post_recorder.get_setting(PostRecorder.LAST_POST_TS)
@@ -173,8 +209,7 @@ def sleep_before_next_toot(bar_title: str, sleep_time: int) -> None:
 
 
 def check_updates() -> None:
-    """Check if there is a newer version of MastodonAmnesia available on
-    PyPI."""
+    """Check if there is a newer version of Tootbot available on PyPI."""
     is_outdated = False
     try:
         is_outdated, pypi_version = check_outdated(
@@ -191,7 +226,7 @@ def check_updates() -> None:
 
 
 def start_main() -> None:
-    """Starts actual main processing using async."""
+    """Start actual main processing using async."""
     asyncio.run(main())
 
 
