@@ -1,12 +1,11 @@
-"""This module contains helper classes and methods to assist with determining
-if a reddit post should be published on Mastodon."""
+"""Helper classes and methods to assist with determining if a reddit post should be published on Mastodon."""
 import configparser
 import sys
 from dataclasses import dataclass
 from logging import DEBUG
 from logging import Formatter
-from logging import getLogger
 from logging import StreamHandler
+from logging import getLogger
 from logging.handlers import TimedRotatingFileHandler
 from typing import Any
 from typing import List
@@ -16,10 +15,11 @@ from typing import TypeVar
 from typing import Union
 
 import aiosqlite
+from minimal_activitypub import __display_name__ as minimal_activitypub_logger
 from typing_extensions import Final
 
-from . import __display_name__
 from . import POST_RECORDER_SQLITE_DB
+from . import __display_name__
 
 PR = TypeVar("PR", bound="PostRecorder")
 ConfigClass = TypeVar("ConfigClass", bound="Configuration")
@@ -30,16 +30,21 @@ logger = getLogger(__display_name__)
 class PostRecorder:
     """Implements logging of reddit posts published to Mastodon and also
     checking against the log of published content to determine if a post would
-    be a duplicate."""
+    be a duplicate.
+    """
 
     LAST_POST_TS: Final[str] = "last-post-timestamp"
 
     def __init__(self: PR, history_db_dir: str = ".") -> None:
+        """Initialise PostRecord instance.
+
+        :param history_db_dir: Location where history db should be stored. Default to current directory (.)
+        """
         self.history_db_file = f"{history_db_dir.rstrip('/')}/{POST_RECORDER_SQLITE_DB}"
         self.history_db: Optional[aiosqlite.Connection] = None
 
     async def db_init(self: PR) -> None:
-        """Initialises DB connection and tables if necessary."""
+        """Initialise DB connection and tables if necessary."""
         self.history_db = await aiosqlite.connect(database=self.history_db_file)
         # Make sure DB tables exist
         await self.history_db.execute(
@@ -67,17 +72,15 @@ class PostRecorder:
         await self.history_db.commit()
 
     async def duplicate_check(self: PR, identifier: str) -> bool:
-        """Checks if identifier can be found in log file of content posted to
+        """Check identifier can be found in log file of content posted to
         Mastodon.
 
-        Arguments:
-            identifier (string):
+        :param identifier:
                 Any identifier we want to make sure has not already been posted.
                 This can be id of reddit post, url of media attachment file to be
                 posted, or checksum of media attachment file.
 
-        Returns:
-            boolean:
+        :returns:
                 False if "identifier" is not in log of content already posted to
                 Mastodon
                 True if "identifier" has been found in log of content.
@@ -120,14 +123,13 @@ class PostRecorder:
         shared_url: Optional[str] = None,
         check_sum: Optional[str] = None,
     ) -> None:
-        """Logs details about reddit posts that have been published.
+        """Log details about reddit posts that have been published.
 
-        Arguments:
-            reddit_id (string):
+        :param reddit_id:
                 Id of post on reddit that was published to Mastodon
-            shared_url (string):
+        :param shared_url:
                 URL of media attachment that was shared on Mastodon
-            check_sum (string):
+        :param check_sum:
                 Checksum of media attachment that was shared on Mastodon.
                 This enables checking for duplicate media even if file has been renamed.
         """
@@ -153,7 +155,7 @@ class PostRecorder:
         self: PR,
         key: str,
     ) -> Any:
-        """Retrieves a setting from database.
+        """Retrieve a setting from database.
 
         :param self: Post Recorded instance
         :param key: Key to setting stored in DB
@@ -180,7 +182,7 @@ class PostRecorder:
         key: str,
         value: Union[int, str, float],
     ) -> None:
-        """Retrieves a setting from database.
+        """Save a setting to database.
 
         :param self: Post Recorded instance
         :param key: Key to setting stored in DB
@@ -202,7 +204,8 @@ class PostRecorder:
         await self.history_db.commit()
 
     async def close_db(self: PR) -> None:
-        """Closes db connection."""
+        """Close db connection."""
+        logger.debug("PostRecorder.close_db()")
         if self.history_db:
             await self.history_db.close()
 
@@ -217,8 +220,7 @@ class Secret:
 
 @dataclass
 class BotConfig:
-    """Dataclass holding configuration values for general behaviour of
-    tootbot."""
+    """Dataclass holding configuration values for general behaviour of tootbot."""
 
     post_recorder: PostRecorder
     delay_between_posts: int
@@ -241,8 +243,7 @@ class RedditReaderConfig:
 
 @dataclass
 class PromoConfig:
-    """Dataclass holding configuration values related to Promotional message
-    settings."""
+    """Dataclass holding configuration values related to Promotional message settings."""
 
     every: int
     message: str
@@ -250,8 +251,7 @@ class PromoConfig:
 
 @dataclass
 class HealthCheckConfig:
-    """Dataclass holding configuration values around Healthchecks
-    monitoring."""
+    """Dataclass holding configuration values around Healthchecks monitoring."""
 
     enabled: bool
     base_url: str
@@ -285,12 +285,12 @@ class MastodonConfig:
     use_caption: bool
     use_tags: bool
     use_backlink: bool
+    link_to_media: bool
 
 
 @dataclass
 class SubredditConfig:
-    """Dataclass to hold configuration settings about the subreddits to be
-    monitored."""
+    """Dataclass to hold configuration settings about the subreddits to be monitored."""
 
     name: str
     tags: str
@@ -300,6 +300,7 @@ class SubredditConfig:
 class Configuration:
     """Dataclass to hold all settings for tootbot."""
 
+    # pylint: disable-msg=too-many-locals
     bot: BotConfig
     subreddits: List[SubredditConfig]
     promo: PromoConfig
@@ -314,8 +315,14 @@ class Configuration:
         config_dir: str,
         debug_log: str,
     ) -> ConfigClass:
-        """Loads aor queries config parameters from user, as necessary."""
+        """Load config parameters from user, as necessary.
 
+        :param config_dir: directory to load config.ini file from.
+        :param debug_log: full path including filename to save debug log messages to
+
+        :returns:
+        Configuration data class with config settings
+        """
         if debug_log:
             file_log_formatter = Formatter(
                 "%(asctime)s %(name)s[%(process)d] %(levelname)s %(message)s",
@@ -329,6 +336,9 @@ class Configuration:
             file_handler.setFormatter(file_log_formatter)
             file_handler.setLevel(DEBUG)
             logger.addHandler(file_handler)
+            mal = getLogger(minimal_activitypub_logger)
+            mal.setLevel(DEBUG)
+            mal.addHandler(file_handler)
 
         # Make sure config file exists
         try:
@@ -370,6 +380,7 @@ class Configuration:
             use_caption=config["Mastodon"].getboolean("UseCaption", fallback=True),
             use_tags=config["Mastodon"].getboolean("UseTags", fallback=True),
             use_backlink=config["Mastodon"].getboolean("UseBacklink", fallback=True),
+            link_to_media=config["Mastodon"].getboolean("MediaLink", fallback=False),
         )
         subreddits = await Configuration._load_subreddits_settings(config)
 
@@ -389,6 +400,7 @@ class Configuration:
     async def _load_subreddits_settings(
         config: configparser.ConfigParser,
     ) -> List[SubredditConfig]:
+        """Load Subreddit configuration options."""
         subreddits = []
         for subreddit, hashtags in config.items("Subreddits"):
             subreddits.append(SubredditConfig(subreddit, hashtags.strip()))
@@ -398,6 +410,7 @@ class Configuration:
     async def _load_healthchecks_settings(
         config: configparser.ConfigParser,
     ) -> HealthCheckConfig:
+        """Load HealthChecks configuration options."""
         health_enabled = False
         if len(config["HealthChecks"]["BaseUrl"]) > 0:
             health_enabled = True
@@ -413,6 +426,7 @@ class Configuration:
         config: configparser.ConfigParser,
         config_dir: str,
     ) -> BotConfig:
+        """Load BotSettings configuration options."""
         hash_tags = []
         if config["BotSettings"]["Hashtags"]:
             # Parse list of hashtags
@@ -430,7 +444,7 @@ class Configuration:
 
     @staticmethod
     async def _set_up_logging(config: configparser.ConfigParser) -> None:
-        # Set-up logging
+        """Configure logging."""
         log_level = "INFO"
         if config["BotSettings"]["LogLevel"]:
             log_level = config["BotSettings"]["LogLevel"]
@@ -446,7 +460,7 @@ class Configuration:
 
 
 def str_to_bool(value: Any) -> bool:
-    """Helper function to convert a string into a boolean value.
+    """Convert a string into a boolean value.
 
     returns: bool
     """
