@@ -1,9 +1,7 @@
-"""This module is a collection of classes and methods to perform the actual
-posting of content to Mastodon."""
+"""Collection of classes and methods to perform the actual posting of content to Mastodon."""
 import asyncio
 import logging
 import os
-import sys
 from dataclasses import dataclass
 from typing import Any
 from typing import AsyncIterator
@@ -17,12 +15,11 @@ import aiohttp
 import magic
 from minimal_activitypub.client_2_server import ActivityPub
 from minimal_activitypub.client_2_server import ActivityPubError
-from minimal_activitypub.client_2_server import ApiError
 from tqdm.asyncio import tqdm
 
-from . import __display_name__
 from . import CLIENT_WEBSITE
 from . import USER_AGENT
+from . import __display_name__
 from .collect import LinkedMediaHelper
 from .collect import MediaAttachment
 from .collect import RedditHelper
@@ -41,7 +38,7 @@ logger = logging.getLogger(__display_name__)
 MP = TypeVar("MP", bound="MastodonPublisher")
 FileManagerClass = TypeVar("FileManagerClass", bound="FileManager")
 
-SECRETS_FILE = "mastodon.secret"  # pragma: allowlist secret
+SECRETS_FILE = "mastodon.secret"  # noqa: S105 - not actually a secret but file name of where secrets are stored
 
 
 @dataclass
@@ -66,27 +63,25 @@ class MastodonPublisher:
         session: aiohttp.ClientSession,
         secrets: Secret,
     ) -> MP:
-        """Initialises and returns an new MastodonPublisher class."""
-        try:
-            instance = ActivityPub(
-                instance=config.mastodon_config.domain,
-                access_token=secrets.client_secret.strip(),
-                session=session,
-            )
-            await instance.determine_instance_type()
+        """Initialise and returns a new MastodonPublisher class."""
+        instance = ActivityPub(
+            instance=config.mastodon_config.domain,
+            access_token=secrets.client_secret.strip(),
+            session=session,
+        )
+        await instance.determine_instance_type()
 
-            user_info = await instance.verify_credentials()
+        user_info = await instance.verify_credentials()
 
-            logger.debug(
-                "Successfully authenticated on %s as @%s",
-                config.mastodon_config.domain,
-                user_info["username"],
-            )
-
-        except (ActivityPubError, ApiError) as error:
-            logger.error("Error while logging into Mastodon: %s", error)
-            logger.error("Tootbot cannot continue, now shutting down")
-            sys.exit(1)
+        print(
+            f"Successfully authenticated as "
+            f"@{user_info['username']} on {config.mastodon_config.domain}"
+        )
+        logger.debug(
+            "Successfully authenticated on %s as @%s",
+            config.mastodon_config.domain,
+            user_info["username"],
+        )
 
         return cls(
             instance=instance,
@@ -99,19 +94,19 @@ class MastodonPublisher:
 
     @staticmethod
     async def get_secrets(mastodon_domain: str, config_dir: str) -> Secret:
-        """Checks that Mastodon API secrets are available. If not collects
-        neccesarry info from user input the create and store APi secrets for
+        """Check that Mastodon API secrets are available. If not collects
+        necessary info from user input the create and store APi secrets for
         Mastodon API secrets.
 
-        Arguments:
-            mastodon_domain: domain name for Mastodon instance used for tooting. This
+        :param mastodon_domain: domain name for Mastodon instance used for tooting. This
             is read from config.ini file, where it must be configured
-            :config_dir: direcotry containing all the configuration files
+        :type mastodon_domain: str
+        :param config_dir: directory containing all the configuration files
+        :type config_dir: str
         """
-
         mastodon_secrets = Secret(  # nosec B106
             client_id=None,
-            client_secret="undefined",  # pragma: allowlist secret
+            client_secret="undefined",  # noqa: S106 - placeholder, not actually a 'password' or secret
         )
 
         secrets_file = f"{config_dir}/{SECRETS_FILE}"
@@ -120,45 +115,57 @@ class MastodonPublisher:
             # If the secret file doesn't exist,
             # it means the setup process hasn't happened yet
             print("Mastodon API keys not found. (See wiki for help).")
-            user_name = input("[ .. ] Enter email address for Mastodon account: ")
-            password = input("[ .. ] Enter password for Mastodon account: ")
-            print("Generating login key for Mastodon...")
-            try:
-                async with aiohttp.ClientSession() as session:
-                    access_token = await ActivityPub.get_auth_token(
+
+            async with aiohttp.ClientSession() as session:
+                # Create app
+                client_id, client_secret = await ActivityPub.create_app(
+                    instance_url=mastodon_domain,
+                    session=session,
+                    client_website=CLIENT_WEBSITE,
+                )
+
+                # Get Authorization Code / URL
+                authorization_request_url = (
+                    await ActivityPub.generate_authorization_url(
                         instance_url=mastodon_domain,
-                        username=user_name,
-                        password=password,
-                        session=session,
+                        client_id=client_id,
                         user_agent=USER_AGENT,
-                        client_website=CLIENT_WEBSITE,
                     )
-
-                    mastodon = ActivityPub(
-                        instance=mastodon_domain,
-                        access_token=access_token,
-                        session=session,
-                    )
-                    await mastodon.determine_instance_type()
-                    user_info = await mastodon.verify_credentials()
-                mastodon_username = user_info["username"]
-                print(
-                    f"Successfully authenticated on "
-                    f"{mastodon_domain} as @{mastodon_username}"
                 )
-                async with aiofiles.open(file=secrets_file, mode="w") as file:
-                    await file.write(access_token)
-                    await file.write("\n")
-                    await file.write(mastodon.instance)
-                    mastodon_secrets.client_secret = access_token
                 print(
-                    f"Mastodon login information now stored in " f"{secrets_file} file"
+                    f"Please go to the following URL and follow the instructions:\n"
+                    f"{authorization_request_url}"
+                )
+                authorization_code = input("[...] Please enter the authorization code:")
+
+                # Validate authorization code and get access token
+                access_token = await ActivityPub.validate_authorization_code(
+                    session=session,
+                    instance_url=mastodon_domain,
+                    authorization_code=authorization_code,
+                    client_id=client_id,
+                    client_secret=client_secret,
                 )
 
-            except (ActivityPubError, ApiError) as error:
-                logger.error("Error while logging into Mastodon: %s", error)
-                logger.error("Tootbot cannot continue, now shutting down")
-                sys.exit(1)
+                # Verify access token works
+                mastodon = ActivityPub(
+                    instance=mastodon_domain,
+                    access_token=access_token,
+                    session=session,
+                )
+                await mastodon.determine_instance_type()
+                user_info = await mastodon.verify_credentials()
+            mastodon_username = user_info["username"]
+            print(
+                f"Successfully authenticated on "
+                f"{mastodon_domain} as @{mastodon_username}"
+            )
+            async with aiofiles.open(file=secrets_file, mode="w") as file:
+                await file.write(access_token)
+                await file.write("\n")
+                await file.write(mastodon_domain)
+                mastodon_secrets.client_secret = access_token
+            print(f"Mastodon login information now stored in " f"{secrets_file} file")
 
         else:
             async with aiofiles.open(
@@ -169,28 +176,39 @@ class MastodonPublisher:
 
         return mastodon_secrets
 
-    async def make_post(
+    async def make_post(  # noqa: C901
         self: MP,
         posts: Dict[Any, Any],
         reddit_helper: RedditHelper,
         media_helper: LinkedMediaHelper,
+        duplicate_checks: bool = True,
     ) -> None:
-        """Makes a post on mastodon from a selection of reddit submissions.
+        """Post status on mastodon from a selection of reddit submissions.
 
-        Arguments:
-            posts: A dictionary of subreddit specific hashtags and PRAW Submission
-                   objects
-            reddit_helper: Helper class to work with Reddit
-            media_helper: Helper class to retrieve media linked to from a reddit
-            Submission.
+        :param posts: A dictionary of subreddit specific hashtags and PRAW Submission objects
+        :type posts: Dict[Any, Any}
+        :param reddit_helper: Helper class to work with Reddit
+        :type reddit_helper: RedditHelper
+        :param media_helper: Helper class to retrieve media linked to from a reddit Submission.
+        :type media_helper: LinkedMediaHelper
+        :param duplicate_checks: flag to determine if duplicate checks should be performed.
+        :type duplicate_checks: bool
         """
+        # pylint: disable=too-many-locals
+        logger.debug(
+            "MastodonPublisher.make_post(posts=%s, "
+            "reddit_helper, "
+            "media_helper, "
+            "duplicate_checks=%s)",
+            posts,
+            duplicate_checks,
+        )
         break_to_mainloop = False
         for additional_hashtags, source_posts in posts.items():
             if break_to_mainloop:
                 break
 
             for post in source_posts:
-
                 # Find out if we have any attachments to include with toot.
                 logger.debug(
                     "MastodonPublisher.make_post - Getting attachments for post %s",
@@ -198,7 +216,8 @@ class MastodonPublisher:
                 )
                 attachments = MediaAttachment(source_posts[post], media_helper)
                 await attachments.get_media_files()
-                await self._remove_posted_earlier(attachments)
+                if duplicate_checks:
+                    await self._remove_posted_earlier(attachments)
 
                 # Make sure the post contains media,
                 # if MEDIA_POSTS_ONLY in config is set to True
@@ -213,10 +232,11 @@ class MastodonPublisher:
                         source_posts[post].id,
                     )
                     # Log the post anyway
-                    await self.bot.post_recorder.log_post(
-                        reddit_id=source_posts[post].id
-                    )
-                    continue
+                    if duplicate_checks:
+                        await self.bot.post_recorder.log_post(
+                            reddit_id=source_posts[post].id,
+                        )
+                        continue
 
                 try:
                     # Generate promo message if needed
@@ -234,7 +254,10 @@ class MastodonPublisher:
                     )
 
                     # Upload media files if available
-                    media_ids = await self._post_attachments(attachments)
+                    media_ids = await self._post_attachments(
+                        attachments=attachments,
+                        duplicate_checks=duplicate_checks,
+                    )
                     if self.media_only and len(media_ids) == 0:
                         # Skip posts where no media has been able to be uploaded
                         # and we've set media_only posts to True
@@ -245,6 +268,11 @@ class MastodonPublisher:
                     if source_posts[post].over_18 and self.nsfw_marked:
                         spoiler = "NSFW"
 
+                    logger.debug(
+                        "MastodonPublisher.make_post"
+                        " - self.mastodon_config.media_always_sensitive=%s",
+                        self.mastodon_config.media_always_sensitive,
+                    )
                     # Post to Mastodon
                     toot = await self.instance.post_status(
                         status=caption,
@@ -257,10 +285,11 @@ class MastodonPublisher:
                     logger.debug("Posted to Mastodon: %s (at %s)", caption, toot["url"])
 
                     # Log the toot
-                    await self.bot.post_recorder.log_post(
-                        reddit_id=source_posts[post].id,
-                        shared_url=source_posts[post].url,
-                    )
+                    if duplicate_checks:
+                        await self.bot.post_recorder.log_post(
+                            reddit_id=source_posts[post].id,
+                            shared_url=source_posts[post].url,
+                        )
 
                     self.num_non_promo_posts += 1
                     self.mastodon_config.number_of_errors = 0
@@ -281,13 +310,18 @@ class MastodonPublisher:
                 break_to_mainloop = True
                 break
 
-    async def _post_attachments(self: MP, attachments: MediaAttachment) -> List[str]:
+    async def _post_attachments(
+        self: MP,
+        attachments: MediaAttachment,
+        duplicate_checks: bool,
+    ) -> List[str]:
         """_post_attachments post any media in attachments.media_paths list.
 
         Arguments:
             attachments: object with a list of paths to media to be posted on Mastodon
 
-        Returns:
+        Returns
+        -------
             media_ids: List of dicts returned by mastodon.media_post
         """
         media_ids: List[str] = []
@@ -313,7 +347,8 @@ class MastodonPublisher:
             )
 
             # Log the media upload
-            await self.bot.post_recorder.log_post(check_sum=checksum)
+            if duplicate_checks:
+                await self.bot.post_recorder.log_post(check_sum=checksum)
         try:
             medias = await asyncio.gather(*tasks)
             for media in medias:
@@ -333,9 +368,9 @@ class MastodonPublisher:
         attachments and removes any from the list that have already been posted
         earlier.
 
-        Arguments:
-            attachments: object with list of paths to media files proposed to be
+        :param attachments: object with list of paths to media files proposed to be
             posted on Mastodon
+        :type attachments: MediaAttachment
         """
         # Build a list of checksums for files that have already been posted earlier
         logger.debug(
@@ -380,6 +415,11 @@ class FileManager:
         file_name: str,
         chunk_size: int = 64 * 1024,
     ) -> None:
+        """Initialise new FileManager instance.
+
+        :param file_name: Name of file
+        :param chunk_size: Size of chunks of data. Defaults to 64kb
+        """
         self.name = file_name
         self.chunk_size = chunk_size
         self.size = os.path.getsize(self.name)
@@ -395,7 +435,8 @@ class FileManager:
 
     async def file_reader(self: FileManagerClass) -> AsyncIterator[bytes]:
         """file_reader returns file contents in chunks of bytes and updates an
-        associated progress bar."""
+        associated progress bar.
+        """
         async with aiofiles.open(self.name, "rb") as file:
             chunk = await file.read(self.chunk_size)
             while chunk:
